/* global oVmap, nsVmap, goog, ol, vitisApp */

/**
 * @author: Anthony Borghi
 * @Description: Fichier contenant la classe nsVmap.nsToolsManager.Urlexporter
 * cette classe permet à l'utilisateur d'exporter une url pour un autre utilisateur
 */
goog.provide('nsVmap.nsToolsManager.Urlexporter');

goog.require('oVmap');

/**
 * @classdesc
 * Class {@link nsVmap.nsToolsManager.VmapUser}
 * @constructor
 * @export
 */
nsVmap.nsToolsManager.Urlexporter = function () {
    oVmap.log('nsVmap.nsToolsManager.Urlexporter');
};



/**
 * Directive
 * @return {angular.Directive} The directive specs.
 * @constructor
 */
nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterDirective = function () {
    oVmap.log("nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterDirective");
    return {
        restrict: 'A',
        scope: {
            'map': '=appMap',
            'lang': '=appLang'
        },
        controller: 'AppUrlExporterController',
        controllerAs: 'ctrl',
        bindToController: true,
        templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/tools/' + (oVmap['properties']['is_mobile'] ? 'urlexporter_mobile.html' : 'urlexporter.html')
    };
};

/**
 * Controler
 * @constructor
 * @param {object} $scope
 * @returns {undefined}
 * @ngInject
 */
nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController = function ($scope, $q, $element) {
    oVmap.log("nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController");

    var this_ = this;

    this.$scope_ = $scope;

    /**
     * The current properties
     */
    this['properties'] = oVmap['properties'];

    /**
     * The current token
     */
    this['token'] = oVmap['properties']['token'];

    /**
     * @type {string}
     */
    this["urlToExport"] = "";
    /**
     * @type {boolean}
     */
    this["usePublicToken"] = false;

    this["urlConstructor"]();

    // Affiche les modales en plein écran pour la version mobile
    //if (oVmap['properties']['is_mobile']) {
    //    $element.find('.modal').on('shown.bs.modal', function () {
    //        $('.modal-backdrop.fade.in').hide();
    //        $('.modal.fade.in').find('.modal-dialog').addClass('mobile-full-modal');
    //    });
    //}

    console.log(this['properties']);

    // recalcule l'url quand on change de carte
    oVmap['scope'].$on('mapChanged', function () {
        this_["urlConstructor"]();
    });

    $scope.$watch("ctrl.usePublicToken", function(){
        this_["urlConstructor"]();
    })

    oVmap.getMap().getOLMap().on("moveend", function(event){
        this_["urlConstructor"]();
    });
};

/**
 * Display the user form in display mode
 * @returns {undefined}
 * @export
 */
nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController.prototype.urlConstructor = function () {
    oVmap.log("nsVmap.nsToolsManager.Urlexporter.urlExporterController.urlConstructor");

    var sUrl = this['properties']['web_server_name'] + "/" + this['properties']['application'];

    if (goog.isDefAndNotNull(this['properties']['environment'])){
        if (this['properties']['environment'] != ""){
            sUrl += "_" + this['properties']['environment'];
        }
    }

    sUrl += "?";

    var oMapCatalog = oVmap.getMapManager().getMapCatalog();
    var iMapId = oMapCatalog["maps"][oMapCatalog["usedMap"]]["map_id"];

    sUrl += "map_id=" + iMapId;

    // si public add token
    if (this["usePublicToken"]){
        sUrl += "&token=" + this['properties']['public_token'];
    }

    // get extent and add in url
    var sExtent = oVmap.getMap().getOLMap().getView().calculateExtent(oVmap.getMap().getOLMap().getSize()).join("|");
    sUrl += "&extent=" + encodeURI(sExtent);

    this["urlToExport"] = sUrl;
};

// Définit la directive et le controller
oVmap.module.directive('appUrlExporter', nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterDirective);
oVmap.module.controller('AppUrlExporterController', nsVmap.nsToolsManager.Urlexporter.prototype.urlExporterController);